@@ -36,7 +36,7 @@ pub fn nil() -> Term { fls() }
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(null(), nil()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(null(), nil()), NOR, 0), tru());
 /// # }
 /// ```
 pub fn null() -> Term {
@@ -70,7 +70,7 @@ pub fn null() -> Term {
 ///                 nil()
 ///             )
 ///         )
-///     ), &Normal, 0
+///     ), NOR, 0
 /// );
 /// let list_110_from_vec = Term::from(vec![one(), one(), zero()]);
 ///
@@ -79,9 +79,9 @@ pub fn null() -> Term {
 /// ```
 pub fn cons() -> Term { pair() }
 
-/// Equivalent to `pair::first()`; applied to a Church list it returns its first element.
+/// Equivalent to `pair::fst()`; applied to a Church list it returns its first element.
 ///
-/// HEAD := FIRST
+/// HEAD := FST
 ///
 /// # Example
 /// ```
@@ -95,15 +95,15 @@ pub fn cons() -> Term { pair() }
 ///
 /// let list_110 = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta(app!(head(), list_110), &Normal, 0), one());
+/// assert_eq!(beta(app!(head(), list_110), NOR, 0), one());
 /// # }
 /// ```
-pub fn head() -> Term { first() }
+pub fn head() -> Term { fst() }
 
-/// Equivalent to `pair::second()`; applied to a Church list it returns a new list with all its
+/// Equivalent to `pair::snd()`; applied to a Church list it returns a new list with all its
 /// elements but the first one.
 ///
-/// TAIL := SECOND
+/// TAIL := SND
 ///
 /// # Example
 /// ```
@@ -117,15 +117,15 @@ pub fn head() -> Term { first() }
 ///
 /// let list_110 = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta(app!(tail(), list_110), &Normal, 0), Term::from(vec![one(), zero()]));
+/// assert_eq!(beta(app!(tail(), list_110), NOR, 0), Term::from(vec![one(), zero()]));
 /// # }
 /// ```
-pub fn tail() -> Term { second() }
+pub fn tail() -> Term { snd() }
 
 /// Applied to a Church list it returns its Church-encoded length.
 ///
-/// LENGTH := Z (λgcx.NULL x (λz.c) (λz.g (SUCC c) (SECOND x)) I) ZERO
-/// = Z (λλλ NULL 1 (λ 3) (λ 4 (SUCC 3) (SECOND 2)) I) ZERO
+/// LENGTH := Z (λzal.NULL l (λx.a) (λx.z (SUCC a) (SND l)) I) ZERO
+/// = Z (λλλ NULL 1 (λ 3) (λ 4 (SUCC 3) (SND 2)) I) ZERO
 ///
 /// # Example
 /// ```
@@ -138,8 +138,8 @@ pub fn tail() -> Term { second() }
 ///
 /// let list_4 = Term::from(vec![1.into(), 1.into(), 0.into(), 1.into()]);
 ///
-/// assert_eq!(beta(app!(length(), nil() ), &Normal, 0), 0.into());
-/// assert_eq!(beta(app!(length(), list_4), &Normal, 0), 4.into());
+/// assert_eq!(beta(app!(length(), nil() ), NOR, 0), 0.into());
+/// assert_eq!(beta(app!(length(), list_4), NOR, 0), 4.into());
 /// # }
 /// ```
 pub fn length() -> Term {
@@ -153,7 +153,7 @@ pub fn length() -> Term {
                 abs(app!(
                     Var(4),
                     app(succ(), Var(3)),
-                    app(second(), Var(2))
+                    app(snd(), Var(2))
                 )),
                 i()
             )
@@ -164,8 +164,8 @@ pub fn length() -> Term {
 
 /// Reverses a Church list.
 ///
-/// REVERSE := Z (λgal.NULL l (λx.a) (λx.g (PAIR (FIRST l) a) (SECOND l) I)) NIL =
-/// Z (λ λ λ NULL 1 (λ 3) (λ 4 (PAIR (FIRST 2) 3) (SECOND 2)) I) NIL
+/// REVERSE := Z (λzal.NULL l (λx.a) (λx.z (PAIR (FST l) a) (SND l) I)) NIL =
+/// Z (λ λ λ NULL 1 (λ 3) (λ 4 (PAIR (FST 2) 3) (SND 2)) I) NIL
 ///
 /// # Example
 /// ```
@@ -179,7 +179,7 @@ pub fn length() -> Term {
 ///
 /// let list = Term::from(vec![one(), one(), zero()]);
 ///
-/// assert_eq!(beta(app!(reverse(), list), &Normal, 0),
+/// assert_eq!(beta(app!(reverse(), list), NOR, 0),
 ///            Term::from(vec![zero(), one(), one()])
 /// );
 /// # }
@@ -194,8 +194,8 @@ pub fn reverse() -> Term {
                 abs(Var(3)),
                 abs(app!(
                     Var(4),
-                    app!(pair(), app(first(), Var(2)), Var(3)),
-                    app(second(), Var(2))
+                    app!(pair(), app(fst(), Var(2)), Var(3)),
+                    app(snd(), Var(2))
                 )),
                 i()
             )
@@ -218,7 +218,7 @@ pub fn reverse() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(list(), 3.into(), 0.into(), 1.into(), 1.into()), &Normal, 0),
+/// assert_eq!(beta(app!(list(), 3.into(), 0.into(), 1.into(), 1.into()), NOR, 0),
 ///            Term::from(vec![0.into(), 1.into(), 1.into()]));
 /// # }
 /// ```
@@ -237,8 +237,8 @@ pub fn list() -> Term {
 
 /// Applied to 2 Church lists it concatenates them.
 ///
-/// APPEND := Z (λgab. NULL a (λx.b) (λx.PAIR (FIRST a) (g (SECOND a) b)) I) =
-/// Z (λ λ λ NULL 2 (λ 2) (λ PAIR (FIRST 3) (4 (SECOND 3) 2)) I)
+/// APPEND := Z (λzab. NULL a (λx.b) (λx.PAIR (FST a) (z (SND a) b)) I) =
+/// Z (λ λ λ NULL 2 (λ 2) (λ PAIR (FST 3) (4 (SND 3) 2)) I)
 ///
 /// # Example
 /// ```
@@ -252,7 +252,7 @@ pub fn list() -> Term {
 /// let list1 = Term::from(vec![0.into(), 1.into()]);
 /// let list2 = Term::from(vec![2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(append(), list1, list2), &Normal, 0),
+/// assert_eq!(beta(app!(append(), list1, list2), NOR, 0),
 ///            Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]));
 /// # }
 /// ```
@@ -265,8 +265,8 @@ pub fn append() -> Term {
                 abs(Var(2)),
                 abs(app!(
                     pair(),
-                    app(first(), Var(3)),
-                    app!(Var(4), app(second(), Var(3)), Var(2))
+                    app(fst(), Var(3)),
+                    app!(Var(4), app(snd(), Var(3)), Var(2))
                 )),
                 i()
             )
@@ -277,7 +277,7 @@ pub fn append() -> Term {
 /// Applied to a Church number `i` and a Church list it returns the `i`-th
 /// (zero-indexed) element of the list.
 ///
-/// INDEX := λix. FIRST (x SECOND i) = λ λ FIRST (2 SECOND 1)
+/// INDEX := λil. FST (l SND i) = λ λ FST (2 SND 1)
 ///
 /// # Example
 /// ```
@@ -290,20 +290,20 @@ pub fn append() -> Term {
 ///
 /// let list = Term::from(vec![3.into(), 4.into(), 5.into()]);
 ///
-/// assert_eq!(beta(app!(index(), 0.into(), list.clone()), &Normal, 0), 3.into());
-/// assert_eq!(beta(app!(index(), 2.into(), list        ), &Normal, 0), 5.into());
+/// assert_eq!(beta(app!(index(), 0.into(), list.clone()), NOR, 0), 3.into());
+/// assert_eq!(beta(app!(index(), 2.into(), list        ), NOR, 0), 5.into());
 /// # }
 /// ```
 pub fn index() -> Term {
     abs(abs(
-        app(first(), app!(Var(2), second(), Var(1)))
+        app(fst(), app!(Var(2), snd(), Var(1)))
     ))
 }
 
 /// Applied to a function and a Church list it maps the function over it.
 ///
-/// MAP := Z (λgfx. NULL x (λa.NIL) (λa.PAIR (f (FIRST x)) (g f (SECOND x))) I) =
-/// Z (λ λ λ NULL 1 (λ NIL) (λ PAIR (3 (FIRST 2)) (4 3 (SECOND 2))) I)
+/// MAP := Z (λzfl. NULL l (λx.NIL) (λx.PAIR (f (FST l)) (z f (SND l))) I) =
+/// Z (λ λ λ NULL 1 (λ NIL) (λ PAIR (3 (FST 2)) (4 3 (SND 2))) I)
 ///
 /// # Example
 /// ```
@@ -317,7 +317,7 @@ pub fn index() -> Term {
 ///
 /// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(map(), succ(), list), &Normal, 0),
+/// assert_eq!(beta(app!(map(), succ(), list), NOR, 0),
 ///            Term::from(vec![2.into(), 3.into(), 4.into()]));
 /// # }
 /// ```
@@ -330,8 +330,8 @@ pub fn map() -> Term {
                 abs(nil()),
                 abs(app!(
                     pair(),
-                    app(Var(3), app(first(), Var(2))),
-                    app!(Var(4), Var(3), app(second(), Var(2)))
+                    app(Var(3), app(fst(), Var(2))),
+                    app!(Var(4), Var(3), app(snd(), Var(2)))
                 )),
                 i()
             )
@@ -343,8 +343,8 @@ pub fn map() -> Term {
 /// [left fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
 /// list.
 ///
-/// FOLDL := Z (λgfex. NULL x (λz.e) (λz.g f (f e (FIRST x)) (SECOND x)) I) =
-/// Z (λ λ λ λ NULL 1 (λ 2) (λ 5 4 (4 3 (FIRST 2)) (SECOND 2)) I)
+/// FOLDL := Z (λzfsl. NULL l (λx.s) (λx.z f (f s (FST l)) (SND l)) I) =
+/// Z (λ λ λ λ NULL 1 (λ 3) (λ 5 4 (4 3 (FST 2)) (SND 2)) I)
 ///
 /// # Example
 /// ```
@@ -358,8 +358,8 @@ pub fn map() -> Term {
 ///
 /// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(foldl(), plus(), 0.into(), list ), &Normal, 0), 6.into());
-/// assert_eq!(beta(app!(foldl(), plus(), 0.into(), nil()), &Normal, 0), 0.into());
+/// assert_eq!(beta(app!(foldl(), plus(), 0.into(), list ), NOR, 0), 6.into());
+/// assert_eq!(beta(app!(foldl(), plus(), 0.into(), nil()), NOR, 0), 0.into());
 /// # }
 /// ```
 pub fn foldl() -> Term {
@@ -372,8 +372,8 @@ pub fn foldl() -> Term {
                 abs(app!(
                     Var(5),
                     Var(4),
-                    app!(Var(4), Var(3), app(first(), Var(2))),
-                    app(second(), Var(2))
+                    app!(Var(4), Var(3), app(fst(), Var(2))),
+                    app(snd(), Var(2))
                 )),
                 i()
             )
@@ -385,8 +385,8 @@ pub fn foldl() -> Term {
 /// [right fold](https://en.wikipedia.org/wiki/Fold_(higher-order_function)#Folds_on_lists) on the
 /// list.
 ///
-/// FOLDR := λfex. Z (λgy. NULL y (λz.e) (λz.f (FIRST y) (g (SECOND y))) I) x =
-/// λ λ λ Z (λ λ NULL 1 (λ 5) (λ 6 (FIRST 2) (3 (SECOND 2))) I) 1
+/// FOLDR := λfsl. Z (λzt. NULL t (λx.s) (λx.f (FST t) (z (SND t))) I) l =
+/// λ λ λ Z (λ λ NULL 1 (λ 5) (λ 6 (FST 2) (3 (SND 2))) I) 1
 ///
 /// # Example
 /// ```
@@ -400,8 +400,8 @@ pub fn foldl() -> Term {
 ///
 /// let list = Term::from(vec![1.into(), 2.into(), 3.into()]);
 ///
-/// assert_eq!(beta(app!(foldr(), plus(), 0.into(), list ), &Normal, 0), 6.into());
-/// assert_eq!(beta(app!(foldr(), plus(), 0.into(), nil()), &Normal, 0), 0.into());
+/// assert_eq!(beta(app!(foldr(), plus(), 0.into(), list ), NOR, 0), 6.into());
+/// assert_eq!(beta(app!(foldr(), plus(), 0.into(), nil()), NOR, 0), 0.into());
 /// # }
 /// ```
 pub fn foldr() -> Term {
@@ -415,8 +415,8 @@ pub fn foldr() -> Term {
                     abs(Var(5)),
                     abs(app!(
                         Var(6),
-                        app(first(), Var(2)),
-                        app(Var(3), app(second(), Var(2)))
+                        app(fst(), Var(2)),
+                        app(Var(3), app(snd(), Var(2)))
                     )),
                     i()
                 )
@@ -428,8 +428,8 @@ pub fn foldr() -> Term {
 
 /// Applied to a predicate and a Church list it filters the list based on the predicate.
 ///
-/// FILTER := Z (λgfx. NULL x (λz.NIL) (λz.f (FIRST x) (PAIR (FIRST x)) I (g f (SECOND x))) I) =
-/// Z (λ λ λ NULL 1 (λ NIL) (λ 3 (FIRST 2) (PAIR (FIRST 2)) I (4 3 (SECOND 2))) I)
+/// FILTER := Z (λzpl. NULL l (λx.NIL) (λx.p (FST l) (PAIR (FST l)) I (z p (SND l))) I) =
+/// Z (λ λ λ NULL 1 (λ NIL) (λ 3 (FST 2) (PAIR (FST 2)) I (4 3 (SND 2))) I)
 ///
 /// # Example
 /// ```
@@ -445,9 +445,9 @@ pub fn foldr() -> Term {
 /// let list = Term::from(vec![0.into(), 1.into(), 2.into(), 3.into()]);
 /// let gt1  = app!(c(), gt(), 1.into());
 ///
-/// assert_eq!(beta(app!(filter(), is_zero(), list.clone()), &Normal, 0),
+/// assert_eq!(beta(app!(filter(), is_zero(), list.clone()), NOR, 0),
 ///            Term::from(vec![0.into()]));
-/// assert_eq!(beta(app!(filter(), gt1, list), &Normal, 0),
+/// assert_eq!(beta(app!(filter(), gt1, list), NOR, 0),
 ///            Term::from(vec![2.into(), 3.into()]));
 /// # }
 /// ```
@@ -460,10 +460,10 @@ pub fn filter() -> Term {
                 abs(nil()),
                 abs(app!(
                     Var(3),
-                    app(first(), Var(2)),
-                    app(pair(), app(first(), Var(2))),
+                    app(fst(), Var(2)),
+                    app(pair(), app(fst(), Var(2))),
                     i(),
-                    app!(Var(4), Var(3), app(second(), Var(2)))
+                    app!(Var(4), Var(3), app(snd(), Var(2)))
                 )),
                 i()
             )
@@ -770,6 +770,7 @@ mod test {
     use super::*;
     use reduction::beta;
     use reduction::Order::*;
+    use arithmetic::{is_zero, plus};
 
     #[test]
     fn empty_list() {
@@ -793,7 +794,7 @@ mod test {
                         nil()
                     )
                 )
-            ), &Normal, 0
+            ), NOR, 0
         );
         assert_eq!(list_pushed, list_consed);
     }
@@ -868,4 +869,27 @@ mod test {
         assert_eq!(list[3], 3.into());
         assert_eq!(list[4], 4.into());
     }
+
+    #[test]
+    fn cbv_list_functions() {
+        let l = Term::from(vec![1.into(), 2.into(), 3.into(), 4.into()]);
+
+        assert_eq!(beta(app!( length(), l.clone()), HAP, 0), 4.into());
+
+        assert_eq!(beta(app!(reverse(), l.clone()), HAP, 0),
+            Term::from(vec![4.into(), 3.into(), 2.into(), 1.into()]));
+
+        assert_eq!(beta(app!(list(), 4.into(), 1.into(), 2.into(), 3.into(), 4.into()), HAP, 0), l);
+
+        assert_eq!(beta(app!(append(), Term::from(vec![1.into(), 2.into()]),
+            Term::from(vec![3.into(), 4.into()])), HAP, 0), l);
+
+        assert_eq!(beta(app!(map(), succ(), l.clone()), HAP, 0),
+            Term::from(vec![2.into(), 3.into(), 4.into(), 5.into()]));
+
+        assert_eq!(beta(app!(foldl(), plus(), 0.into(), l.clone()), HAP, 0), 10.into());
+        assert_eq!(beta(app!(foldr(), plus(), 0.into(), l.clone()), HAP, 0), 10.into());
+
+        assert_eq!(beta(app!(filter(), is_zero(), l.clone()), HAP, 0), Term::from(vec![]));
+    }
 }
