@@ -19,7 +19,7 @@ pub const SHOW_REDUCTIONS: bool = false;
 /// - The `CBN` order reduces to weak head normal form
 /// - The `CBV` order reduces to weak normal form
 /// - The `HSP` order reduces to head normal form
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Clone, Copy)]
 pub enum Order {
     /// Normal - leftmost outermost; the most popular lambda calculus reduction strategy
     NOR,
@@ -66,33 +66,33 @@ fn _apply(lhs: &mut Term, rhs: &Term, depth: usize) {
         } else if i > depth {
             *lhs = Var(i - 1) // decrement a free variable's index
         },
-        Abs(_) => {
-            _apply(lhs.unabs_ref_mut().unwrap(), rhs, depth + 1)
+        Abs(ref mut abstracted) => {
+            _apply(abstracted, rhs, depth + 1)
         },
-        App(_, _) => {
-            _apply(lhs.lhs_ref_mut().unwrap(), rhs, depth);
-            _apply(lhs.rhs_ref_mut().unwrap(), rhs, depth)
+        App(ref mut lhs_lhs, ref mut lhs_rhs) => {
+            _apply(lhs_lhs, rhs, depth);
+            _apply(lhs_rhs, rhs, depth)
         }
     }
 }
 
 fn update_free_variables(term: &mut Term, added_depth: usize, own_depth: usize) {
     match *term {
-        Var(i) => if i > own_depth {
-            *term = Var(i + added_depth)
+        Var(ref mut i) => if *i > own_depth {
+            *i += added_depth
         },
-        Abs(_) => {
-            update_free_variables(term.unabs_ref_mut().unwrap(), added_depth, own_depth + 1)
+        Abs(ref mut abstracted) => {
+            update_free_variables(abstracted, added_depth, own_depth + 1)
         },
-        App(_, _) => {
-            update_free_variables(term.lhs_ref_mut().unwrap(), added_depth, own_depth);
-            update_free_variables(term.rhs_ref_mut().unwrap(), added_depth, own_depth)
+        App(ref mut lhs_lhs, ref mut lhs_rhs) => {
+            update_free_variables(lhs_lhs, added_depth, own_depth);
+            update_free_variables(lhs_rhs, added_depth, own_depth)
         }
     }
 }
 
 /// Performs β-reduction on a `Term` with the specified evaluation `Order` and an optional limit on
-/// number of reductions (`0` means no limit).
+/// number of reductions (`0` means no limit); it returns the `Term` after reductions.
 ///
 /// # Example
 ///
@@ -131,40 +131,14 @@ pub fn beta(mut term: Term, order: Order, limit: usize) -> Term {
 /// // hybrid applicative: 39
 /// ```
 pub fn benchmark(term: &Term, exclude: &[Order]) {
-    let mut count = 0;
-    if !exclude.contains(&CBN) {
-        term.clone().beta_cbn(0, 0, &mut count);
-        println!("{}:       {}", CBN, count);
-        count = 0;
-    }
-    if !exclude.contains(&NOR) {
-        term.clone().beta_nor(0, 0, &mut count);
-        println!("{}:             {}", NOR, count);
-        count = 0;
-    }
-    if !exclude.contains(&CBV) {
-        term.clone().beta_cbv(0, 0, &mut count);
-        println!("{}:      {}", CBV, count);
-        count = 0;
-    }
-    if !exclude.contains(&APP) {
-        term.clone().beta_app(0, 0, &mut count);
-        println!("{}:        {}", APP, count);
-        count = 0;
-    }
-    if !exclude.contains(&HSP) {
-        term.clone().beta_hsp(0, 0, &mut count);
-        println!("{}:         {}", HSP, count);
-        count = 0;
-    }
-    if !exclude.contains(&HNO) {
-        term.clone().beta_hno(0, 0, &mut count);
-        println!("{}:      {}", HNO, count);
-        count = 0;
-    }
-    if !exclude.contains(&HAP) {
-        term.clone().beta_hap(0, 0, &mut count);
-        println!("{}: {}", HAP, count);
+    println!("benchmarking β-reduction strategies for {}:\n", term);
+    for order in [CBN, NOR, CBV, APP, HSP, HNO, HAP].into_iter() {
+        if !exclude.contains(order) {
+            println!("{}:{}{}{}", order,
+                " ".repeat(if !SHOW_REDUCTIONS { 19 - format!("{}", order).len() } else { 1 }),
+                term.clone().beta(*order, 0), if SHOW_REDUCTIONS { "\n" } else { "" }
+            )
+        }
     }
 }
 
@@ -205,32 +179,25 @@ impl Term {
     }
 
     fn eval_with_info(&mut self, depth: u32, count: &usize) {
-        if SHOW_REDUCTIONS {
-            print!("\n{}. {}\n=>", count + 1, show_precedence(self, 0, depth));
-            let mut indent_len = ((*count + 1) as f32).log10().trunc() as usize + 3;
-            if DISPLAY_CLASSIC { indent_len += 3 }
-            let lhs = self.lhs_ref().unwrap();
-            let rhs = self.rhs_ref().unwrap();
-            if lhs.unabs_ref().unwrap().unvar_ref().is_ok() {
-                if DISPLAY_CLASSIC { indent_len += 3 } else { indent_len += 2 }
-                if rhs.unapp_ref().is_ok() { indent_len += 1 }
-            }
-
-            for _ in 0..indent_len { print!(" ") };
-        };
+        if SHOW_REDUCTIONS { print!("\n{}. {}\n", count + 1, show_precedence(self, 0, depth)) }
 
         let copy = self.clone();
-        *self = copy.eval().unwrap();
+        *self = copy.eval().unwrap(); // safe; only called in reduction sites
 
-        if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) }
+        if SHOW_REDUCTIONS {
+            let mut indent_len = ((*count + 1) as f32).log10().trunc() as usize + 3;
+            if DISPLAY_CLASSIC { indent_len += 3 }
+            println!("=>{}{}", " ".repeat(indent_len), show_precedence(self, 0, depth))
+        }
     }
 
     fn is_reducible(&self, limit: usize, count: &usize) -> bool {
-        self.lhs_ref().unwrap().unabs_ref().is_ok() && (limit == 0 || *count < limit )
+        self.lhs_ref().and_then(|t| t.unabs_ref()).is_ok() && (limit == 0 || *count < limit )
     }
 
     /// Performs β-reduction on a `Term` with the specified evaluation `Order` and an optional
-    /// limit on number of reductions (`0` means no limit).
+    /// limit on number of reductions (`0` means no limit); it returns the number of performed
+    /// reductions.
     ///
     /// # Example
     ///
@@ -243,9 +210,9 @@ impl Term {
     ///
     /// assert_eq!(pred_one, 0.into());
     /// ```
-    pub fn beta(&mut self, order: Order, limit: usize) {
+    pub fn beta(&mut self, order: Order, limit: usize) -> usize {
         if SHOW_REDUCTIONS {
-            println!("reducing {} [{} order{}]:", self, order,
+            println!("β-reducing {} [{} order{}]:", self, order,
                 if limit != 0 {
                     format!(", limit of {} reduction{}", limit, if limit == 1 { "" } else { "s" })
                 } else {
@@ -265,10 +232,13 @@ impl Term {
             HNO => self.beta_hno(0, limit, &mut count),
             HAP => self.beta_hap(0, limit, &mut count)
         }
+
         if SHOW_REDUCTIONS {
             println!("\nresult after {} reduction{}: {}\n", count,
-                if limit == 1 { "" } else { "s" }, self);
+                if count == 1 { "" } else { "s" }, self);
         };
+
+        count
     }
 
     fn beta_cbn(&mut self, depth: u32, limit: usize, count: &mut usize) {
@@ -292,8 +262,7 @@ impl Term {
         if limit != 0 && *count == limit { return }
 
         match *self {
-            Var(_) => (),
-            Abs(_) => self.unabs_ref_mut().unwrap().beta_nor(depth + 1, limit, count),
+            Abs(ref mut abstracted) => abstracted.beta_nor(depth + 1, limit, count),
             App(_, _) => {
                 self.lhs_ref_mut().unwrap().beta_cbn(depth, limit, count);
 
@@ -305,7 +274,8 @@ impl Term {
                     self.lhs_ref_mut().unwrap().beta_nor(depth, limit, count);
                     self.rhs_ref_mut().unwrap().beta_nor(depth, limit, count);
                 }
-            }
+            },
+            _ => ()
         }
     }
 
@@ -331,8 +301,7 @@ impl Term {
         if limit != 0 && *count == limit { return }
 
         match *self {
-            Var(_) => (),
-            Abs(_) => self.unabs_ref_mut().unwrap().beta_app(depth + 1, limit, count),
+            Abs(ref mut abstracted) => abstracted.beta_app(depth + 1, limit, count),
             App(_, _) => {
                 self.lhs_ref_mut().unwrap().beta_app(depth, limit, count);
                 self.rhs_ref_mut().unwrap().beta_app(depth, limit, count);
@@ -342,7 +311,8 @@ impl Term {
                     *count += 1;
                     self.beta_app(depth, limit, count);
                 }
-            }
+            },
+            _ => ()
         }
     }
 
@@ -350,8 +320,7 @@ impl Term {
         if limit != 0 && *count == limit { return }
 
         match *self {
-            Var(_) => (),
-            Abs(_) => self.unabs_ref_mut().unwrap().beta_hap(depth + 1, limit, count),
+            Abs(ref mut abstracted) => abstracted.beta_hap(depth + 1, limit, count),
             App(_, _) => {
                 self.lhs_ref_mut().unwrap().beta_cbv(depth, limit, count);
                 self.rhs_ref_mut().unwrap().beta_hap(depth, limit, count);
@@ -363,7 +332,8 @@ impl Term {
                 } else {
                     self.lhs_ref_mut().unwrap().beta_hap(depth, limit, count);
                 }
-            }
+            },
+            _ => ()
         }
     }
 
@@ -371,8 +341,7 @@ impl Term {
         if limit != 0 && *count == limit { return }
 
         match *self {
-            Var(_) => (),
-            Abs(_) => self.unabs_ref_mut().unwrap().beta_hsp(depth + 1, limit, count),
+            Abs(ref mut abstracted) => abstracted.beta_hsp(depth + 1, limit, count),
             App(_, _) => {
                 self.lhs_ref_mut().unwrap().beta_hsp(depth, limit, count);
 
@@ -381,7 +350,8 @@ impl Term {
                     *count += 1;
                     self.beta_hsp(depth, limit, count)
                 }
-            }
+            },
+            _ => ()
         }
     }
 
@@ -389,8 +359,7 @@ impl Term {
         if limit != 0 && *count == limit { return }
 
         match *self {
-            Var(_) => (),
-            Abs(_) => self.unabs_ref_mut().unwrap().beta_hno(depth + 1, limit, count),
+            Abs(ref mut abstracted) => abstracted.beta_hno(depth + 1, limit, count),
             App(_, _) => {
                 self.lhs_ref_mut().unwrap().beta_hsp(depth, limit, count);
 
@@ -402,7 +371,8 @@ impl Term {
                     self.lhs_ref_mut().unwrap().beta_hno(depth, limit, count);
                     self.rhs_ref_mut().unwrap().beta_hno(depth, limit, count);
                 }
-            }
+            },
+            _ => ()
         }
     }
 }
