@@ -73,7 +73,22 @@ impl Term {
     /// ```
     pub fn unvar_ref(&self) -> Result<&usize, Error> {
         match *self {
-            Var(ref n) => Ok(&n),
+            Var(ref n) => Ok(n),
+            _ => Err(NotAVar)
+        }
+    }
+
+    /// Returns a mutable reference to a variable's index.
+    ///
+    /// # Example
+    /// ```
+    /// use lambda_calculus::term::Term::*;
+    ///
+    /// assert_eq!(Var(1).unvar_ref_mut(), Ok(&mut 1));
+    /// ```
+    pub fn unvar_ref_mut(&mut self) -> Result<&mut usize, Error> {
+        match *self {
+            Var(ref mut n) => Ok(n),
             _ => Err(NotAVar)
         }
     }
@@ -105,7 +120,7 @@ impl Term {
     /// ```
     pub fn unabs_ref(&self) -> Result<&Term, Error> {
         match *self {
-            Abs(ref x) => Ok(&*x),
+            Abs(ref x) => Ok(x),
             _ => Err(NotAnAbs)
         }
     }
@@ -121,7 +136,7 @@ impl Term {
     /// ```
     pub fn unabs_ref_mut(&mut self) -> Result<&mut Term, Error> {
         match *self {
-            Abs(ref mut x) => Ok(&mut *x),
+            Abs(ref mut x) => Ok(x),
             _ => Err(NotAnAbs)
         }
     }
@@ -151,7 +166,7 @@ impl Term {
     /// ```
     pub fn unapp_ref(&self) -> Result<(&Term, &Term), Error> {
         match *self {
-            App(ref lhs, ref rhs) => Ok((&*lhs, &*rhs)),
+            App(ref lhs, ref rhs) => Ok((lhs, rhs)),
             _ => Err(NotAnApp)
         }
     }
@@ -166,7 +181,7 @@ impl Term {
     /// ```
     pub fn unapp_ref_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
         match *self {
-            App(ref mut lhs, ref mut rhs) => Ok((&mut *lhs, &mut *rhs)),
+            App(ref mut lhs, ref mut rhs) => Ok((lhs, rhs)),
             _ => Err(NotAnApp)
         }
     }
